@@ -20,7 +20,7 @@ import time
 from time import perf_counter
 import torch
 from torch import Tensor
-import torch.distributed as dist
+from deepspeed import comm as dist
 from torch.nn import Module, ModuleList
 import torch.nn.functional as F
 
@@ -80,12 +80,20 @@ def gumbel_rsample(shape: Tuple, device: torch.device) -> Tensor:
     return gumbel(shape)
 
 
+from deepspeed import comm as dist
+
+# einsum dimensions: (g)roup, (s)equence, (e)xpert, (m)odel, (c)apacity
+# See https://arxiv.org/pdf/2006.16668.pdf for details.
+
+
 # Based on https://github.com/pytorch/pytorch/pull/40762
 class _AllToAll(torch.autograd.Function):
     @staticmethod
-    def forward(ctx: Any,
-                group: dist.ProcessGroup,
-                input: Tensor) -> Tensor:  # type: ignore
+    def forward(
+            ctx: Any,
+            # TODO: replace with DS process group
+            group: torch.distributed.ProcessGroup,
+            input: Tensor) -> Tensor:  # type: ignore
         ctx.group = group
         input = input.contiguous()
         output = torch.empty_like(input)
@@ -206,7 +214,7 @@ def top1gating(logits: Tensor,
     # if we don't want to drop any tokens
     if not drop_tokens:
         new_capacity = torch.max(exp_counts).to(logits.device)
-        dist.all_reduce(new_capacity, op=dist.ReduceOp.MAX, group=dist.group.WORLD)
+        dist.all_reduce(new_capacity, op=dist.ReduceOp.MAX, group=dist.get_world_group())
         capacity = new_capacity
 
     # Compute l_aux
@@ -269,10 +277,11 @@ def top1gating(logits: Tensor,
 
 
 def top2gating(logits: Tensor,
-               capacity_factor: float) -> Tuple[Tensor,
-                                                Tensor,
-                                                Tensor,
-                                                Tensor]:
+               capacity_factor: float,
+               min_capacity: int) -> Tuple[Tensor,
+                                           Tensor,
+                                           Tensor,
+                                           Tensor]:
     """Implements Top2Gating on logits."""
     # everything is in fp32 in this function
     gates = F.softmax(logits, dim=1)
@@ -418,7 +427,8 @@ class TopKGate(Module):
         else:
             gate_output = top2gating(
                 logits,
-                self.capacity_factor if self.training else self.eval_capacity_factor)
+                self.capacity_factor if self.training else self.eval_capacity_factor,
+                self.min_capacity)
 
         if self.wall_clock_breakdown:
             self.timers('TopKGate').stop()
