@@ -4,7 +4,7 @@ Copyright 2021 The Microsoft DeepSpeed Team
 import types
 import torch
 import numpy as np
-import torch.distributed as dist
+from deepspeed import comm as dist
 from torch._utils import _flatten_dense_tensors, _unflatten_dense_tensors
 
 
@@ -305,7 +305,7 @@ class OnebitLamb(torch.optim.Optimizer):
                             torch.zeros(self.server_chunk_sizes[i],
                                         device=self.exp_avg_flat[i].device))
                         torch.cuda.empty_cache()
-                        if torch.distributed.get_rank() == 0:
+                        if dist.get_rank() == 0:
                             print("Cupy Buffers Initialized Successfully.")
 
                         self.comm_backend_handle.compressed_allreduce(
@@ -314,7 +314,7 @@ class OnebitLamb(torch.optim.Optimizer):
                             self.server_errors[0],
                             self.deepspeed.local_rank)
 
-                        if torch.distributed.get_rank() == 0:
+                        if dist.get_rank() == 0:
                             print('Pop out errors', flush=True)
                         del self.worker_errors[:]
                         del self.server_errors[:]
@@ -389,9 +389,7 @@ class OnebitLamb(torch.optim.Optimizer):
         if not self.initialize:
             self.lamb_freeze_key = False
             self.initialize = True
-            print(
-                f"Finished the initialization step at rank {torch.distributed.get_rank()}"
-            )
+            print(f"Finished the initialization step at rank {dist.get_rank()}")
             return loss
 
         if self.lamb_freeze_key is False:
@@ -427,7 +425,7 @@ class OnebitLamb(torch.optim.Optimizer):
         del self.corrected_tensor_sizes[:]
         del self.server_chunk_sizes[:]
         if self.state[self.param_groups[0]['params'][0]]['step'] < self.freeze_step:
-            if torch.distributed.get_rank() == 0:
+            if dist.get_rank() == 0:
                 print("Checkpoint loaded and OnebitLamb warmup stage starts/continues.")
             if self.lamb_freeze_key is True:
                 self.lamb_freeze_key = False
@@ -442,7 +440,7 @@ class OnebitLamb(torch.optim.Optimizer):
                     if 'scaling_coeff' in self.state[p]:
                         self.state[p].pop('scaling_coeff')
         else:
-            if torch.distributed.get_rank() == 0:
+            if dist.get_rank() == 0:
                 print(
                     "Checkpoint loaded and OnebitLamb compression stage starts/continues."
                 )
